package org.seasar.struts.extension.taglib;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspException;

import org.apache.struts.taglib.TagUtils;
import org.seasar.framework.container.SingletonS2Container;
import org.seasar.framework.convention.NamingConvention;
import org.seasar.struts.extension.util.SerializeUtilz;
import org.seasar.struts.taglib.S2FormTag;
import org.seasar.struts.util.S2ActionMappingUtil;

/**
 * {@link S2FormTag} の拡張クラスです。
 *
 * @author awaawa
 *
 */
public class XFormTag extends S2FormTag {

	/** serialVersionUID。 */
	private static final long serialVersionUID = 1L;

	/**
	 * {@inheritDoc}
	 * {@link NamingConvention#getViewRootPath()}　を付与します。
	 * 差分変更ができないため、
	 * {@link S2FormTag#renderAction(StringBuffer)}
	 * の処理をコピーして、一部分を修正しています。sastrutsのバージョンアップ時に注意が必要になります。
	 */
	@Override
	protected void renderAction(StringBuffer results) {
		HttpServletRequest request = (HttpServletRequest) pageContext
				.getRequest();
		HttpServletResponse response = (HttpServletResponse) pageContext
				.getResponse();
		results.append(" action=\"");
		String contextPath = request.getContextPath();
		StringBuffer value = new StringBuffer();
		if (contextPath.length() > 1) {
			value.append(contextPath);
		}

		NamingConvention namingConvention
				= SingletonS2Container.getComponent(NamingConvention.class);
		String viewRootPath = namingConvention.getViewRootPath();

		value.append(viewRootPath);
		value.append(action);
		results.append(response.encodeURL(value.toString()));
		results.append("\"");
	}

	/**
	 * {@inheritDoc}
	 * actionForm がシリアライズ可能な場合、
	 * リクエストパラメータにBase64エンコードされた ActionForm の直列化情報を
	 * inputタグ(hidden)に設定します。
	 */
	@Override
	public int doEndTag() throws JspException {

		StringBuffer results = new StringBuffer();

		results.append(super.renderToken());

		Object form = S2ActionMappingUtil.getActionMapping().getActionForm();

		String value = SerializeUtilz.fromObjectToString(form);

		if (value != null) {
			TagUtils tagUtils = TagUtils.getInstance();

			String name = S2ActionMappingUtil.getActionMapping().getAttribute();

			results.append("<input type=\"hidden\" name=\"");
			results.append(tagUtils.filter(name));
			results.append("\" value=\"");
			results.append(tagUtils.filter(value));

			if (TagUtils.getInstance().isXhtml(this.pageContext)) {
				results.append("\" />");
			} else {
				results.append("\">");
			}
		}

		TagUtils.getInstance().write(pageContext, results.toString());

		return super.doEndTag();
	}
}
